@@ -1,10 +1,14 @@
 export const JWT_TOKEN_TYPES = {
   REFRESH: 'refresh',
   ACCESS: 'access',
+  PATIENT_PORTAL_ACCESS: 'patient_portal_access',
+  PATIENT_PORTAL_REFRESH: 'patient_portal_refresh',
 };
 
 export const SYSTEM_USER_UUID = '00000000-0000-0000-0000-000000000000';
 
+export const ADMIN_USER_EMAIL = 'admin@tamanu.io';
+
 export const CAN_ACCESS_ALL_FACILITIES = 'ALL';
 
 // When adding more scopes here, you need to consider:
